/*
 *  The C source output plugin for Tiled, (c) 2005
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Avelino Herrera Morales <avelinoherrera@hotpop.com>
 */

package tiled.plugins.c;

import java.io.*;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Stack;

import tiled.io.MapWriter;
import tiled.io.PluginLogger;
import tiled.core.*;

/**
 * An exporter for C header files
 */
public class CMapWriter implements MapWriter
{
    /**
     * Loads a map from a file.
     *
     * @param filename the filename of the map file
     */
    public void writeMap(Map map, String filename) throws Exception {
        writeMap(map, new FileOutputStream(filename));
    }

    /**
     * Loads a tileset from a file.
     *
     * @param filename the filename of the tileset file
     */
    public void writeTileset(TileSet set, String filename) throws Exception {
        System.out.println("Tilesets are not supported!");
        System.out.println("(asked to write " + filename + ")");
    }

    public void writeMap(Map map, OutputStream out) throws Exception {
        PrintStream pout = new PrintStream(out);
        ListIterator it = map.getLayers();
        while (it.hasNext()) {
            TileLayer layer = (TileLayer) it.next();
            String prefix = layer.getName().replace(' ', '_');
            pout.println("#define  " + prefix + "_WIDTH  " + layer.getWidth());
            pout.println("#define  " + prefix + "_HEIGHT " + layer.getHeight());
            pout.println();
            pout.println("const unsigned char " + prefix + "_map [" + layer.getWidth() + " * " + layer.getHeight() + "] = {");
            boolean first = true;
            int layerWidth = layer.getWidth();
            int layerHeight = layer.getHeight();
            for (int y = 0; y < layerHeight; y++) {
                pout.print("\t");
                for (int x = 0; x < layerWidth; x++) {
                    Tile tile = layer.getTileAt(x, y);
                    if (tile == null)
                        pout.print(-1);
                    else
                        pout.print(tile.getId());
                    if ((x < (layerWidth - 1)) || (y < (layerHeight - 1)))
                        pout.print(", ");
                }
                pout.println();
            }
            pout.println("};");
            pout.println();
            pout.println();
        }
    }

    public void writeTileset(TileSet set, OutputStream out) throws Exception {
        System.out.println("Tilesets are not supported!");
    }

    /**
     * @see tiled.io.MapReader#getFilter()
     */
    public String getFilter() throws Exception {
        return "*.h";
    }

    public String getDescription() {
        return
            "+----------------------------------+\n" +
            "|  An exporter for C header files  |\n" +
            "| (c) 2005 Avelino Herrera Morales |\n" +
            "|     avelinoherrera@hotpop.com    |\n" +
            "+----------------------------------+";
    }

    public String getPluginPackage() {
        return "The C source files export plugin";
    }

    public String getName() {
        return "The C source exporter";
    }

    public boolean accept(File pathname) {
        try {
            String path = pathname.getCanonicalPath().toLowerCase();
            if (path.endsWith(".h")) {
                return true;
            }
        } catch (IOException e) {}
        return false;
    }

    public void setErrorStack(Stack es) {
        // TODO: implement setErrorStack
    }

	public void setLogger(PluginLogger logger) {
		// TODO: implement setLogger
	}
}
